@@ -787,6 +787,11 @@ SessionStoreService.prototype = {
         tabs.push(tabData);
         continue;
       }
+      else if (browser.parentNode.__SS_data && browser.parentNode.__SS_data._tab) {
+        // use the data to be restored when the tab hasn't been completely loaded
+        tabs.push(browser.parentNode.__SS_data);
+        continue;
+      }
       var history = null;
       
       try {
@@ -880,6 +885,13 @@ SessionStoreService.prototype = {
       }
     }
     catch (ex) { debug(ex); } // POSTDATA is tricky - especially since some extensions don't get it right
+
+    var ownerURI =
+      (aEntry instanceof Ci.nsISHEntry_MOZILLA_1_8_BRANCH2) ?
+        aEntry.ownerURI : null;
+    if (ownerURI) {
+        entry.ownerURI = ownerURI.spec;
+    }
     
     if (!(aEntry instanceof Ci.nsISHContainer)) {
       return entry;
@@ -912,9 +924,13 @@ SessionStoreService.prototype = {
                                   wrappedTextarea.name;
     if (!id
       || !(wrappedTextarea instanceof Ci.nsIDOMHTMLTextAreaElement 
-      || wrappedTextarea instanceof Ci.nsIDOMHTMLInputElement)) {
+      || wrappedTextarea instanceof Ci.nsIDOMHTMLInputElement && wrappedTextarea.type != "password")) {
       return false; // nothing to save
     }
+    if (/^(?:\d+\|)+/.test(id)) {
+      // text could be restored into a subframe, so skip it (see bug 463206)
+      return false;
+    }
     
     if (!aPanel.__SS_text) {
       aPanel.__SS_text = [];
@@ -979,7 +995,8 @@ SessionStoreService.prototype = {
     Array.forEach(aWindow.getBrowser().browsers, function(aBrowser, aIx) {
       try {
         var tabData = this._windows[aWindow.__SSi].tabs[aIx];
-        if (tabData.entries.length == 0)
+        if (tabData.entries.length == 0 ||
+            aBrowser.parentNode.__SS_data && aBrowser.parentNode.__SS_data._tab)
           return; // ignore incompletely initialized tabs
         
         var text = [];
@@ -1029,7 +1046,10 @@ SessionStoreService.prototype = {
       }
     }
     
-    this._windows[aWindow.__SSi].tabs.forEach(function(aTabData) { aTabData.entries.forEach(extractHosts); });
+    this._windows[aWindow.__SSi].tabs.forEach(function(aTabData) {
+      if (aTabData.entries instanceof Array) // cf. bug 402349
+        aTabData.entries.forEach(extractHosts);
+    });
   },
 
   /**
@@ -1060,6 +1080,7 @@ SessionStoreService.prototype = {
                 value += cookie.isDomain ? "domain=" + cookie.rawHost + ";" : "";
                 value += cookie.path ? "path=" + cookie.path + ";" : "";
                 value += cookie.isSecure ? "secure;" : "";
+                value += cookie.isHttpOnly ? "httponly;" : "";
               }
             }
             if (value) {
@@ -1287,13 +1308,22 @@ SessionStoreService.prototype = {
       }
     }
     
-    // mark the tabs as loading (at this point about:blank
-    // has completed loading in all tabs, so it won't interfere)
+    // mark the tabs as loading
     for (t = 0; t < aTabs.length; t++) {
+      if (!aTabs[t].entries || !aTabs[t].entries[0])
+        continue; // there won't be anything to load
+      
       var tab = aTabs[t]._tab;
+      var browser = tabbrowser.getBrowserForTab(tab);
+      browser.stop(); // in case about:blank isn't done yet
+      
       tab.setAttribute("busy", "true");
       tabbrowser.updateIcon(tab);
       tabbrowser.setTabTitleLoading(tab);
+      
+      // keep the data around to prevent dataloss in case
+      // a tab gets closed before it's been properly restored
+      browser.parentNode.__SS_data = aTabs[t];
     }
     
     // make sure to restore the selected tab first (if any)
@@ -1401,7 +1431,7 @@ SessionStoreService.prototype = {
    */
   _deserializeHistoryEntry: function sss_deserializeHistoryEntry(aEntry, aIdMap) {
     var shEntry = Cc["@mozilla.org/browser/session-history-entry;1"].
-                  createInstance(Ci.nsISHEntry);
+                  createInstance(Ci.nsISHEntry_MOZILLA_1_8_BRANCH2);
     
     var ioService = Cc["@mozilla.org/network/io-service;1"].
                     getService(Ci.nsIIOService);
@@ -1438,6 +1468,10 @@ SessionStoreService.prototype = {
       stream.setData(aEntry.postdata, -1);
       shEntry.postData = stream;
     }
+
+    if (aEntry.ownerURI) {
+        shEntry.ownerURI = ioService.newURI(aEntry.ownerURI, null, null);
+    }
     
     if (aEntry.children && shEntry instanceof Ci.nsISHContainer) {
       for (var i = 0; i < aEntry.children.length; i++) {
@@ -1457,10 +1491,16 @@ SessionStoreService.prototype = {
       return;
     }
     
+    // XSS note: always call this before injecting content into a document!
+    function hasExpectedURL(aDocument, aURL) {
+      return !aURL || aURL.replace(/#.*/, "") == aDocument.location.href.replace(/#.*/, "");
+    }
+    
     var textArray = this.__SS_restore_text ? this.__SS_restore_text.split(" ") : [];
-    function restoreTextData(aContent, aPrefix) {
+    function restoreTextData(aContent, aPrefix, aURL) {
       textArray.forEach(function(aEntry) {
-        if (/^((?:\d+\|)*)(#?)([^\s=]+)=(.*)$/.test(aEntry) && (!RegExp.$1 || RegExp.$1 == aPrefix)) {
+        if (/^((?:\d+\|)*)(#?)([^\s=]+)=(.*)$/.test(aEntry) &&
+            RegExp.$1 == aPrefix && hasExpectedURL(aContent.document, aURL)) {
           var document = aContent.document;
           var node = RegExp.$2 ? document.getElementById(RegExp.$3) : document.getElementsByName(RegExp.$3)[0] || null;
           if (node && "value" in node) {
@@ -1474,28 +1514,37 @@ SessionStoreService.prototype = {
       });
     }
     
+    var window = this.ownerDocument.defaultView;
     function restoreTextDataAndScrolling(aContent, aData, aPrefix) {
-      restoreTextData(aContent, aPrefix);
+      restoreTextData(aContent, aPrefix, aData.url);
       if (aData.innerHTML) {
-        aContent.setTimeout(function(aHTML) { if (this.document.designMode == "on") { this.document.body.innerHTML = aHTML; } }, 0, aData.innerHTML);
+        window.setTimeout(function() {
+          if (aContent.document.designMode == "on" &&
+              hasExpectedURL(aContent.document, aData.url)) {
+            aContent.document.body.innerHTML = aData.innerHTML;
+          }
+        }, 0);
       }
       if (aData.scroll && /(\d+),(\d+)/.test(aData.scroll)) {
         aContent.scrollTo(RegExp.$1, RegExp.$2);
       }
       for (var i = 0; i < aContent.frames.length; i++) {
-        if (aData.children && aData.children[i]) {
-          restoreTextDataAndScrolling(aContent.frames[i], aData.children[i], i + "|" + aPrefix);
+        if (aData.children && aData.children[i] &&
+          hasExpectedURL(aContent.document, aData.url)) {
+          restoreTextDataAndScrolling(aContent.frames[i], aData.children[i], aPrefix + i + "|");
         }
       }
     }
     
-    var content = XPCNativeWrapper(aEvent.originalTarget).defaultView;
-    if (this.currentURI.spec == "about:config") {
-      // unwrap the document for about:config because otherwise the properties
-      // of the XBL bindings - as the textbox - aren't accessible (see bug 350718)
-      content = content.wrappedJSObject;
+    if (hasExpectedURL(aEvent.originalTarget, this.__SS_restore_data.url)) {
+      var content = XPCNativeWrapper(aEvent.originalTarget).defaultView;
+      if (this.currentURI.spec == "about:config") {
+        // unwrap the document for about:config because otherwise the properties
+        // of the XBL bindings - as the textbox - aren't accessible (see bug 350718)
+        content = content.wrappedJSObject;
+      }
+      restoreTextDataAndScrolling(content, this.__SS_restore_data, "");
     }
-    restoreTextDataAndScrolling(content, this.__SS_restore_data, "");
     
     // notify the tabbrowser that this document has been completely restored
     var event = this.ownerDocument.createEvent("Events");
@@ -1592,7 +1641,7 @@ SessionStoreService.prototype = {
     
     for (var i = 1; i <= aCookies.count; i++) {
       try {
-        cookieService.setCookieString(ioService.newURI(aCookies["domain" + i], null, null), null, aCookies["value" + i] + "expires=0;", null);
+        cookieService.setCookieStringFromHttp(ioService.newURI(aCookies["domain" + i], null, null), null, null, aCookies["value" + i] + "expires=0", null, null);
       }
       catch (ex) { debug(ex); } // don't let a single cookie stop recovering (might happen if a user tried to edit the session file)
     }
@@ -1948,7 +1997,11 @@ SessionStoreService.prototype = {
       else if (aObj == null) {
         parts.push("null");
       }
-      else if (aObj instanceof Array) {
+      // if it looks like an array, treat it as such -
+      // this is required for all arrays from a sandbox
+      else if (aObj instanceof Array ||
+               typeof aObj == "object" && "length" in aObj &&
+               (aObj.length === 0 || aObj[aObj.length - 1] !== undefined)) {
         parts.push("[");
         for (var i = 0; i < aObj.length; i++) {
           jsonIfy(aObj[i]);
@@ -1961,6 +2014,9 @@ SessionStoreService.prototype = {
       else if (typeof aObj == "object") {
         parts.push("{");
         for (var key in aObj) {
+          if (key == "_tab")
+            continue; // XXXzeniko we might even want to drop all private members
+          
           jsonIfy(key.toString());
           parts.push(":");
           jsonIfy(aObj[key]);
