@@ -1,6 +1,6 @@
-//@line 40 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 40 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 
-//@line 42 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 42 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 var helpBrowser;
 var helpSearchPanel;
 var emptySearch;
@@ -12,16 +12,16 @@ var helpGlossaryPanel;
 var strBundle;
 var gTocDSList = "";
 
-//@line 54 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 54 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 const NC = "http://home.netscape.com/NC-rdf#";
 const MAX_LEVEL = 40; // maximum depth of recursion in search datasources.
 const MAX_HISTORY_MENU_ITEMS = 6;
 
-//@line 68 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 68 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 const platform = "unix";
-//@line 72 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 72 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 
-//@line 74 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 74 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 const RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
     .getService(Components.interfaces.nsIRDFService);
 const RDF_ROOT = RDF.GetResource("urn:root");
@@ -48,13 +48,9 @@ var RE;
 
 var helpFileURI;
 var helpFileDS;
-//@line 102 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 102 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 var helpBaseURI;
 
-var gIgnoreFocus = false;
-var gClickSelectsAll;
-var gIgnoreClick = false;
-
 /* defaultTopic is either set
    1. in the openHelp() call, passed as an argument to the Help window and
       evaluated in init(), or
@@ -66,7 +62,7 @@ var defaultTopic;
 
 const NSRESULT_RDF_SYNTAX_ERROR = 0x804e03f7;
 
-//@line 123 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 119 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function displayTopic(topic) {
     // Get the page to open.
     var uri = getLink(topic);
@@ -79,7 +75,7 @@ function displayTopic(topic) {
       loadURI(uri);
 }
 
-//@line 136 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 132 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function init() {
   // Cache panel references.
   helpSearchPanel = document.getElementById("help-search-panel");
@@ -88,10 +84,16 @@ function init() {
   helpGlossaryPanel = document.getElementById("help-glossary-panel");
   helpBrowser = document.getElementById("help-content");
 
+  // Turn off unnecessary features for security
+  helpBrowser.docShell.allowJavascript = false;
+  helpBrowser.docShell.allowPlugins = false;
+  helpBrowser.docShell.allowSubframes = false;
+  helpBrowser.docShell.allowMetaRedirects = false;
+
   strBundle = document.getElementById("bundle_help");
   emptySearchText = strBundle.getString("emptySearchText");
 
-  initFindBar();
+  gFindBar.initFindBar();
 
   // Get the content pack, base URL, and help topic
   var helpTopic = defaultTopic;
@@ -126,8 +128,9 @@ function init() {
 
   webProgress.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
 
-  gClickSelectsAll = getBoolPref("browser.urlbar.clickSelectsAll", true);
-  
+  var searchBox = document.getElementById("findText");
+  searchBox.clickSelectsAll = getBoolPref("browser.urlbar.clickSelectsAll", true);
+
   setTimeout(focusSearch, 0);
 }
 
@@ -159,7 +162,7 @@ function hideSearchSidebar(aEvent) {
   tableOfContents.removeAttribute("hidden");
 }
 
-//@line 219 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 222 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function loadHelpRDF() {
   if (!helpFileDS) {
     try {
@@ -234,12 +237,12 @@ function loadHelpRDF() {
   }
 }
 
-//@line 295 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 298 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function filterDatasourceByPlatform(aDatasource) {
   filterNodeByPlatform(aDatasource, RDF_ROOT, 0);
 }
 
-//@line 301 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 304 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function filterNodeByPlatform(aDatasource, aCurrentResource, aCurrentLevel) {
   if (aCurrentLevel > MAX_LEVEL) {
      log("Datasources over " + MAX_LEVEL + " levels deep are unsupported.");
@@ -256,7 +259,7 @@ function filterNodeByPlatform(aDatasource, aCurrentResource, aCurrentLevel) {
   }
 }
 
-//@line 320 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 323 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function filterSeqByPlatform(aDatasource, aNode, aCurrentLevel) {
   // get nc:subheading children into an enumerator
   var RDFC = Components.classes["@mozilla.org/rdf/container;1"]
@@ -289,7 +292,7 @@ function filterSeqByPlatform(aDatasource, aNode, aCurrentLevel) {
   }
 }
 
-//@line 354 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 357 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function normalizeLinks(helpBaseURI, links) {
   if (!helpBaseURI) {
     return links;
@@ -331,7 +334,7 @@ function getLink(ID) {
     return null;
 }
 
-//@line 397 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 400 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function getHelpFileURI() {
     return helpFileURI;
 }
@@ -366,33 +369,6 @@ function goBack() {
   }
 }
 
-/* copied from browser.js */
-function BrowserReloadWithFlags(reloadFlags) {
-    /* First, we'll try to use the session history object to reload so 
-     * that framesets are handled properly. If we're in a special 
-     * window (such as view-source) that has no session history, fall 
-     * back on using the web navigation's reload method.
-     */
-
-    var webNav = getWebNavigation();
-    try {
-      var sh = webNav.sessionHistory;
-      if (sh)
-        webNav = sh.QueryInterface(Components.interfaces.nsIWebNavigation);
-    } catch (e) {
-    }
-
-    try {
-      webNav.reload(reloadFlags);
-    } catch (e) {
-    }
-}
-
-function reload() {
-  const reloadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
-  return BrowserReloadWithFlags(reloadFlags);
-}
-
 function goForward() {
     try
     {
@@ -570,39 +546,12 @@ function onselect_loadURI(tree) {
     }// when switching between tabs a spurious row number is returned.
 }
 
-// copied from browser.js to handle the searchbar
-function SearchbarFocusHandler(aEvent, aElt)
-{
-  if (gIgnoreFocus)
-    gIgnoreFocus = false;
-  else if (gClickSelectsAll)
-    aElt.select();
-}
-
-function SearchbarMouseDownHandler(aEvent, aElt)
-{
-  if (aElt.hasAttribute("focused")) {
-    gIgnoreClick = true;
-  } else {
-    gIgnoreFocus = true;
-    gIgnoreClick = false;
-    aElt.setSelectionRange(0, 0);
-  }
-}
-
-function SearchbarClickHandler(aEvent, aElt)
-{
-  if (!gIgnoreClick && gClickSelectsAll && aElt.selectionStart == aElt.selectionEnd)
-    aElt.select();
-}
-
-
 function focusSearch() {
   var searchBox = document.getElementById("findText");
   searchBox.focus();
 }
 
-//@line 669 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 618 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function doFind() {
     if (document.getElementById("help-search-sidebar").hidden)
       showSearchSidebar();
@@ -752,7 +701,7 @@ function getLiteralValue(literal, defaultValue) {
     return null;
 }
 
-//@line 819 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 768 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function log(aText) {
     CONSOLE_SERVICE.logStringMessage(aText);
 }
@@ -769,7 +718,7 @@ function getBoolPref (aPrefname, aDefault)
   }
 }
 
-//@line 836 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/components/help/content/help.js"
+//@line 785 "/builds/tinderbox/Fx-Mozilla1.8-release/Linux_2.4.21-53.EL_Depend/mozilla/toolkit/components/help/content/help.js"
 function getXulWin()
 {
   window.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
